;(function () {
'use strict'

/* Manual tests:

- Open 2 windows. Toggle in one window should affect the other only if they are on same domain.
- Open preferences, change whitelist. All windows should reflect changes (icons and prefs).

*/

function whitelisted(tab) { return settings.whitelist.includes(domain(tab.url)) }

function updateIcon(tab) {
    // I'm guessing tab.url case-folds and normalizes the same way as HTMLAnchorElement.href,
    // but would be nice if I could find documentation to that effect.
    const enabled = ! whitelisted(tab)
    browser.browserAction.setIcon({
        // tabId is an unfortunate name. Because there's only one of these buttons browser per
        // window, not per tab, it only comes into play when you have multiple windows
        tabId: tab.id,
        path: enabled
            ? {
              16: "icons/ksx16.png",
              32: "icons/ksx32.png",
              64: "icons/ksx64.png" }
            : {
              16: "icons/ksxdisabled16.png",
              32: "icons/ksxdisabled32.png",
              64: "icons/ksxdisabled64.png" }
        })
    // Screen reader will read the title (Window Narrator, at least)
    browser.browserAction.setTitle({
        tabId: tab.id,
        title: enabled
            ? 'Keyboard Savior Xtreme'
            : '(Disabled) Keyboard Savior Xtreme'
        })
}
function updateLocation () {
    browser.tabs.query({active: true})
    // Only the focused window receives tab update and activated events, so need to enumerate
    // active tabs in all open windows so that the icon state reflects the change
    .then(tabs => {
        for (let t of tabs) {
            updateIcon(t)
            // No need to filter this message to active tabs, but this is simplest 
            browser.tabs.sendMessage(t.id, whitelisted(t))
        }})
}
updateLocation()
// Copying the example https://github.com/mdn/webextensions-examples/tree/master/bookmark-it
browser.tabs.onUpdated.addListener(updateLocation) // url changes
browser.tabs.onActivated.addListener(updateLocation) // tab switching
onSettingsChange(updateLocation)

browser.browserAction.onClicked.addListener(tab => {
    // todo: toggling a domain disassociates it with its comment, if any
    const loc = domain(tab.url)
    if (loc == null) {
        // No effect on a pages like about:config or ftp://
        return
    }
    if (settings.whitelist.includes(loc)) {
        settings.whitelist = settings.whitelist.filter(d => {
            // Domains in whitelist normalized on save. Could appear more than once if the user put
            // it in settings more than once
            return d !== loc
        })
    } else {
        settings.whitelist.push(loc)
    }
    browser.storage.local.set(settings) // triggers icon state update via prefs listeners
})

})()
